use {
    color_eyre::{eyre::ContextCompat, Report, Result},
    std::io::stdin,
};

macro_rules! numxt_impl {
    ($self:expr, $rest:ident $(,s: $s:pat)? $(,e: $e:pat)?) => {{
        let self_ = $self;
        let (n, rest): (u8, _) = loop {
            break match self_.0 {
                [$($s,)? b'o', b'n', b'e', $($e)?] => (1, $rest),
                [$($s,)? b't', b'w', b'o', $($e)?] => (2, $rest),
                [$($s,)? b't', b'h', b'r', b'e', b'e', $($e)?] => (3, $rest),
                [$($s,)? b'f', b'o', b'u', b'r', $($e)?] => (4, $rest),
                [$($s,)? b'f', b'i', b'v', b'e', $($e)?] => (5, $rest),
                [$($s,)? b's', b'i', b'x', $($e)?] => (6, $rest),
                [$($s,)? b's', b'e', b'v', b'e', b'n', $($e)?] => (7, $rest),
                [$($s,)? b'e', b'i', b'g', b'h', b't', $($e)?] => (8, $rest),
                [$($s,)? b'n', b'i', b'n', b'e', $($e)?] => (9, $rest),
                [$($s,)? c, $($e)?] if c.is_ascii_digit() => (c - 0x30, $rest),
                [$($s,)? _, $($e)?] => {
                    self_.0 = $rest;
                    continue;
                }
                [] => return None,
            };
        };

        self_.0 = rest;
        Some(u64::from(n))
    }};
}

struct NumXt<'a>(&'a [u8]);
impl<'a> DoubleEndedIterator for NumXt<'a> {
    fn next_back(&mut self) -> Option<Self::Item> {
        numxt_impl!(self, rest, s: rest @ ..)
    }
}

impl<'a> Iterator for NumXt<'a> {
    type Item = u64;

    fn next(&mut self) -> Option<Self::Item> {
        numxt_impl!(self, rest, e: rest @ ..)
    }
}

fn part1(x: &str) -> impl Iterator<Item = u64> + DoubleEndedIterator + '_ {
    x.chars().filter_map(|x| x.to_digit(10)).map(u64::from)
}

fn part2(x: &str) -> impl Iterator<Item = u64> + DoubleEndedIterator + '_ {
    NumXt(x.as_bytes())
}

fn main() -> Result<()> {
    //let part = part1;
    let part = part2;

    Ok(println!(
        "{}",
        stdin()
            .lines()
            .map(|x| {
                x.map_err(Report::from).and_then(|x| {
                    let mut it = part(&x);
                    let fst = it.next().context("No first number found")?;
                    Ok(fst * 10 + it.next_back().unwrap_or(fst))
                })
            })
            .try_fold::<_, _, Result<u64>>(0, |a, b: Result<u64>| Ok(a + b?))?
    ))
}
